__author__ = 'wolkens'

import math #, pygame

class ColoringFunction():
    def __init__(self):
        self.cfArray = [[0 for col in range(256)] for sp in range(201)]
        # Create array for 256 color values by 0.01 resolution scalarproduct in range (-1, +1)
        for col in range(256): # Initialize with actual colors
            for sp in range(201):
                self.cfArray[sp][col] = coloringfunction( (sp/100.0 - 1.0) * 0.999, col, 0, 0, 0)[0]

    def coloringfunction(self, sp, r, g, b, l): # Replaces original coloringfunction()
        spx = math.trunc((sp+1)*100) # Calculate array index
        red = self.cfArray[spx][r]
        green = self.cfArray[spx][g]
        blue = self.cfArray[spx][b]
        line = self.cfArray[spx][l]
        return (red, green, blue, line)  # Return converted color

def interpolate(x1 = 0.0, fx1 = 0.0, x2 = 1.0, fx2 = 1.0, x = 0.5):  # Linear interpolation
    return (fx1 + (x - x1) / (x2 - x1) * (fx2 - fx1))

def rgb_to_hex(red, green, blue): # Return color as #rrggbb for the given color values
    hexvalue = '0x%02x%02x%02xff' % (red, green, blue)
    return hexvalue

def coloringfunction(sp, r, g, b, l):  # Shift current color between white and black
    # based on scalar product between viewing vector and normal vector
    # l is line value (line has same r, g and b)
    lighten = [1.00, 1.00, 1.00, 1.00, 1.00, 1.00, 1.00, 1.00, 0.80, 0.50, 0.20]
    darken  = [0.90, 0.80, 0.70, 0.60, 0.50, 0.40, 0.30, 0.25, 0.20, 0.15, 0.10]
    if sp < 0:  # Darken to half RGB
        sp = -sp # Invert
        spx = math.trunc(sp * 10)  # Convert to 0..9
        spdx = sp * 10 - spx  # Calculate fraction part
        modder = interpolate(0, darken[spx], 1, darken[spx + 1], spdx)
        red = math.trunc(r * modder)  # Darken the color
        green = math.trunc(g * modder)
        blue = math.trunc(b * modder)
        line = math.trunc(l * modder)
    else:  # Lighten, maximum to white
        spx = math.trunc(sp * 10)  # Convert to 0..9
        spdx = sp * 10 - spx  # Calculate fraction part
        modder = interpolate(0.0, lighten[spx], 1, lighten[spx + 1], spdx)
        red = math.trunc(255 - (255 - r) * modder) # Lighten the color
        green = math.trunc(255 - (255 - g) * modder)
        blue = math.trunc(255 - (255 - b) * modder)
        line = 255  # No change in line color when lightening
    return (red, green, blue, line)  # Return converted color
