import random, math

class Levels():
    def  __init__(self):
        self.left =  [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0] # No signal on left channel
        self.right = [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0] # No signal on right channel
        self.levelstr = "SG0000000000000000000000000000X" # Default is no signal (2*7 values)

    def setmode(self, m):
        self.mode = m # "random" or "serial"

    def maxlevel(self):
        sumlevelsL, sumlevelsR = 0, 0
        for lvl in range (0, 12):
            sumlevelsL += math.exp(self.left[lvl] / 100)
            sumlevelsR += math.exp(self.right[lvl] / 100)
        maxl = 100 * math.log(sumlevelsL / 7) # Average level for left channel
        maxr = 100 * math.log(sumlevelsR / 7) # Average level for right channel
        return (maxl, maxr)

    def getlevels(self, ch):
        if ch == "L":
            return self.left
        else:
            return self.right

    def setstring(self, str):
        self.levelstr = str

    def update(self):
        if self.mode == "random":
            for freq in range(0, 12):
                self.left[freq]  = random.randint(0, 255)
                self.right[freq] = random.randint(0, 255)
        else: # Use actual data
            for i in range(0, 7): # Actual results: 63, 160, 400, 1000, 2500, 6250, 16000 Hz
                self.left[i] =  int(self.levelstr[ 2+2*i :  4+2*i], 16)
                self.right[i] = int(self.levelstr[16+2*i : 18+2*i], 16)
            # Interpolate the frequencies in between
            self.left[7]   = (self.left[0] + self.left[1]) / 2 # 63, 160 -> 100 Hz
            self.left[8]   = (self.left[1] + self.left[2]) / 2 # 160, 400 -> 250 Hz
            self.left[9]   = (self.left[2] + self.left[3]) / 2 # 400, 1000 -> 630 Hz
            self.left[10]  = (self.left[4] + self.left[5]) / 2 # 2500, 6250 -> 4000 Hz
            self.left[11]  = (self.left[5] + self.left[6]) / 2 # 6250, 16000 -> 10000 Hz
            self.right[7]  = (self.right[0] + self.right[1]) / 2 # 63, 160 -> 100 Hz
            self.right[8]  = (self.right[1] + self.right[2]) / 2 # 160, 400 -> 250 Hz
            self.right[9]  = (self.right[2] + self.right[3]) / 2 # 400, 1000 -> 630 Hz
            self.right[10] = (self.right[4] + self.right[5]) / 2 # 2500, 6250 -> 4000 Hz
            self.right[11] = (self.right[5] + self.right[6]) / 2 # 6250, 16000 -> 10000 Hz
