#!/usr/bin/env python

import os, sys, datetime, pygame, serial, math, random
from pygame.locals import *
from pygame.compat import geterror

import const, myglobals, image, levels, dodecahedron, triangle

def waitsecrollover():
    dt1 = datetime.datetime.now().second
    dt2 = datetime.datetime.now().second
    # print("Timestamp: ", dt1)
    while dt2 == dt1:
        dt2 = datetime.datetime.now().second
    # print("Rollover to: ", dt2)

def main():
    print ("Initializing...")
    myglobals.init()
    print ("Loading, please wait...")
    pygame.init()
    screen = pygame.display.set_mode((const.SCREEN_WIDTH, const.SCREEN_HEIGHT), pygame.FULLSCREEN) # Create screen
    # screen = pygame.display.set_mode((const.SCREEN_WIDTH, const.SCREEN_HEIGHT)) # Create screen
    image.images = image.Images() # Load all images

    myuart = serial.Serial("/dev/ttyAMA0", 38400, timeout=1) # Initialize UART
    outStr = "SGX" # Get data command
    inStr = ""
    myuart.flushInput()
    myuart.flushOutput()

    mylevels = levels.Levels() # Create levels object to store level data
    mylevels.setmode("serial") # Data are coming from the serial port
    LeftStar = dodecahedron.StellatedDodecahedron("L", -400.0, 160.0) # Create dodecahedrons
    RightStar = dodecahedron.StellatedDodecahedron("R", 600.0, 160.0)

    alfa, beta = 0, 0
    axisA = triangle.Point(1/math.sqrt(2), 1/math.sqrt(2), 0)
    axisB = triangle.Point(-1/math.sqrt(2), 1/math.sqrt(2), 0)
    running = True # Run until keypress
    realtick = 0.0

    dt1 = datetime.datetime.now().second
    ctr = 0
    frames = 0.0
    waitsecrollover()

    deg2rad = 3.1416 /360
    mindelta = 0.02
    dax, day, daz = 0.00, 0.00, 0.00
    dbx, dby, dbz = 0.00, 0.00, 0.00

    while running:
        myuart.write(outStr) # Get data command
        mylevels.update() # Get new levels
        screen.blit(image.images.background, (0,0), None, special_flags = 0) # Create the background
        LeftStar.update(screen, mylevels.getlevels("L"), alfa, axisA, 640, 360) # Update dodecahedrons
        RightStar.update(screen, mylevels.getlevels("R"), beta, axisB, 640, 360)

        # Modify rotation axes
        dalfa, dbeta = mylevels.maxlevel() # Rotation speed depends on maximum level
        dalfa, dbeta = dalfa/4000, dbeta/4000 # Constantly rotate
        alfa = alfa + dalfa
        if alfa > 6.29: alfa = alfa - 6.29
        beta = beta + dbeta
        if beta > 6.29: beta = beta - 6.29
        rtckrad = realtick * deg2rad 
        dax = dax + mindelta * math.sin(3.0 * rtckrad)
        day = day + mindelta * math.sin(4.0 * rtckrad)
        daz = daz + mindelta * math.sin(7.0 * rtckrad)
        axisA.x, axisA.y, axisA.z = axisA.x + dax, axisA.y + day, axisA.z + daz
        alen = math.sqrt(axisA.x*axisA.x + axisA.y*axisA.y + axisA.z*axisA.z)
        axisA.x, axisA.y, axisA.z = axisA.x / alen, axisA.y / alen, axisA.z / alen
        dbx = dbx + mindelta * math.sin(6.0 * rtckrad)
        dby = dby + mindelta * math.sin(5.0 * rtckrad)
        dbz = dbz + mindelta * math.sin(3.5 * rtckrad)
        axisB.x, axisB.y, axisB.z = axisB.x + dbx, axisB.y + dby, axisB.z + dbz
        blen = math.sqrt(axisB.x*axisB.x + axisB.y*axisB.y + axisB.z*axisB.z)
        axisB.x, axisB.y, axisB.z = axisB.x / blen, axisB.y / blen, axisB.z / blen
        realtick = realtick + 1.0
        if realtick > 360:
            realtick = realtick - 360

        # Finish the loop
        pygame.display.flip()

        inStr = myuart.read(myuart.inWaiting())
        mylevels.setstring(inStr)

	frames = frames + 1.0
	dt2 = datetime.datetime.now().second
	if dt1 != dt2:
            ctr = ctr + 1
	    dt1 = dt2
	    if ctr == 10:
		running = False

        for event in pygame.event.get():
            if event.type == pygame.KEYDOWN:
                running = False

    pygame.quit()
    print("Result: ",frames/10.0," fps")
    sys.exit()

if __name__ == '__main__':
    main()
